require "pathname"
require "faraday"
require "openssl"

module BBC
  module Cosmos
    module Tools
      class API
        def initialize(host, key_path)
          @connection = Faraday.new host, :ssl => {
            :client_key  => client_key(key_path),
            :client_cert => client_cert(key_path),
            :verify      => false,
            :version     => "TLSv1"
          }
        end

        def client_key(path)
          OpenSSL::PKey::RSA.new(File.read path)
        end

        def client_cert(path)
          OpenSSL::X509::Certificate.new(File.read path)
        end

        def get(path)
          @connection.get path
        end

        def put(path, payload, content_type = "application/json")
          content_type_from content_type
          @connection.put path, payload
        end

        def post(path, payload, content_type = "application/json")
          content_type_from content_type
          @connection.post path, payload
        end

        def content_type_from(type)
          @connection.headers[:content_type] = type
        end
      end
    end
  end
end
