require "json"
require "pathname"
require "bbc/cosmos/tools/commands/base"
require "bbc/cosmos/tools/cosmos_configuration"
require "bbc/cosmos/tools/api"

module BBC
  module Cosmos
    module Tools
      module Commands
        class Config < Base
          attr_accessor :cosmos_config

          def initialize(args = [], local_options = {}, thor_config = {})
            super(args, local_options, thor_config)
            @cosmos_config = BBC::Cosmos::Tools::CosmosConfiguration.new(config)
          end

          desc "generate [COMPONENT]", "Generates config for the given component optionally in the cosmos format"
          method_option :output, :type => :string, :desc => "Output type", :default => "normal"
          method_option :write, :type => :boolean, :desc => "write output to file", :default => false
          method_option :cosmos_format, :type => :boolean, :desc => "Toogles if the output should be in the cosmos format"
          def generate(component)
            if options[:write]
              File.open("#{component}.json", "w") do |f|
                f.write(JSON.pretty_generate(cosmos_config.component(component)))
              end
            end

            if options[:output] == "machine"
              say JSON.pretty_generate(cosmos_config.cosmos_component(component)), :white

              exit 0
            end

            begin

              say "#{banner(component)}\n"

              if options[:cosmos_format]
                say JSON.pretty_generate(cosmos_config.cosmos_component(component)), :white
              else
                say JSON.pretty_generate(cosmos_config.component(component)), :white
              end
            rescue Exception => e
              error e.message
            end
          end

          desc "list", "Lists all components for a given project"
          method_option :output, :type => :string, :desc => "Output type", :default => "normal"
          def list
            if options[:output] == "machine"
              cosmos_config.component_keys.each do |component_id|
                puts "#{component_id}"
              end

              exit 0
            end

            begin

              say banner
              say "\n#{get_key_value('Components', '')}\n"

              cosmos_config.component_keys.each do |component_id|
                say "* #{component_id}", :white
              end
            rescue Exception => e
              error e.message
            end
          end

          desc "push [COMPONENT_ID = nil]", "Push config for a component to the cosmos API"
          method_option :force, :type => :boolean, :desc => "This skips asking of you want to push the changes"
          def push(component_id = nil)
            components = component_id.nil? ? cosmos_config.component_keys : [component_id]
            say "#{banner}\n"

            reply = yes?("Are you sure you want to push changes for #{components.length} components(s) to #{options[:env]}?", :yellow) unless options[:force]
            if reply || reply.nil?

              components.each do |id|
                say "\n#{get_key_value('Component', id)}\n"

                component_config = JSON.generate(cosmos_config.cosmos_component id)

                @api = BBC::Cosmos::Tools::API.new(config.app["api"], options[:key_path])
                response = @api.put sprintf(config.app["config_endpoint"], options[:env], id), component_config

                if response.success?
                  say "Successfully saved", :green
                else
                  api_error response
                end
              end

            else
              error "Action cancelled"
            end

          rescue Exception => e
            error e.message
          end
        end
      end
    end
  end
end
