require "yaml"
require "bbc/cosmos/tools/config"

module BBC
  module Cosmos
    module Tools
      class CosmosConfiguration
        attr_accessor :config

        def initialize(config)
          @config = config
        end

        def component_keys
          config.components.keys
        end

        def component(id)
          config.components[id].tap do |o|
            fail("Invalid component id: #{id}") if o.nil?
          end
        end

        def cosmos_component(id)
          component(id).reduce([]) do |object, (key, value)|
            object.tap do |o|
              o << {
                :value  => secure_key?(value) ? value["value"] : value,
                :key    => key,
                :secure => secure_key?(value)
              }
            end
          end
        end

        private

        def secure_key?(value)
          value.is_a?(Hash) && value["secure"]
        end

      end
    end
  end
end
