require "bbc/cosmos/tools/cloudformation/generator"
require "json"

module BBC
  module Cosmos
    module Tools
      module Types
        class BaseType
          def initialize(id, options, config)
            @id      = id
            @options = options
            @config  = config
          end

          def generate_data(to_json = false)
            if valid_component_and_stack
              get_parameters.each do |key, value|
                if data.has_key?("Parameters") && data["Parameters"][key]
                  data["Parameters"][key]["Default"] = value
                end
              end
              to_json ? JSON.pretty_generate(data) : data
            end
          end

          protected

          def data
            fail NotImplementedError
          end

          def construct_file_path(type)
            File.join(@config.base_path, "stacks", @id, "#{@options[:stack]}.#{type}")
          end

          private

          def valid_component_and_stack
            Tools::Cloudformation::Generator.component_exists?(@id, @config) &&
              Tools::Cloudformation::Generator.stack_exists?(@id, @config, @options[:stack])
          end

          def get_parameters
            @config.resources["cloudformation"]["components"][@id][@options[:stack]]["variants"][@options[:variant]] || []
          end
        end
      end
    end
  end
end
